Package: spdep
Version: 1.4-2
Date: 2026-02-13
Title: Spatial Dependence: Weighting Schemes, Statistics
Encoding: UTF-8
Authors@R: c(person("Roger", "Bivand", role = c("cre", "aut"), 
           email = "Roger.Bivand@nhh.no",
           comment=c(ORCID="0000-0003-2392-6140", ROR="04v53s997")),
	person("Micah", "Altman", role = "ctb"),
	person("Luc", "Anselin", role = "ctb"),
	person("Renato", "Assunção", role = "ctb"),
	person("Anil", "Bera", role = "ctb"),
	person("Olaf", "Berke", role = "ctb"),
	person("F. Guillaume", "Blanchet", role = "ctb"),
	person("Marilia", "Carvalho", role = "ctb"),
	person("Bjarke", "Christensen", role = "ctb"),
	person("Yongwan", "Chun", role = "ctb"),
	person("Carsten", "Dormann", role = "ctb"),
	person("Stéphane", "Dray", role = "ctb"),
        person("Dewey", "Dunnington", role = c("ctb"),
           comment = c(ORCID = "0000-0002-9415-4582")),
	person("Virgilio", "Gómez-Rubio", role = "ctb"),
	person("Malabika", "Koley", role = "ctb"),
	person("Tomasz", "Kossowski", role = "ctb",
           comment = c(ORCID = "0000-0002-9976-4398")),
	person("Elias", "Krainski", role = "ctb"),
	person("Pierre", "Legendre", role = "ctb"),
	person("Nicholas", "Lewin-Koh", role = "ctb"),
	person("Angela", "Li", role = "ctb"),
	person("Giovanni", "Millo", role = "ctb"),
	person("Werner", "Mueller", role = "ctb"),
	person("Hisaji", "Ono", role = "ctb"),
	person("Josiah", "Parry", role = "ctb",
           comment = c(ORCID = "0000-0001-9910-865X")),
	person("Pedro", "Peres-Neto", role = "ctb"),
	person("Michał", "Pietrzak", role = "ctb",
           comment = c(ORCID = "0000-0002-9263-4478")),
	person("Gianfranco", "Piras", role = "ctb"),
	person("Markus", "Reder", role = "ctb"),
	person("Jeff", "Sauer", role = "ctb"),
	person("Michael", "Tiefelsdorf", role = "ctb"),
	person("René", "Westerholt", role="ctb"),
	person("Justyna", "Wilk", role = "ctb",
           comment = c(ORCID = "0000-0003-1495-2910")),
	person("Levi", "Wolf", role = "ctb"),
	person("Danlin", "Yu", role = "ctb"))
Depends: R (>= 3.3.0), methods, spData (>= 2.3.1), sf
Imports: stats, deldir, boot (>= 1.3-1), graphics, utils, grDevices,
        units, s2, e1071, sp (>= 1.0)
Suggests: spatialreg (>= 1.2-1), Matrix, parallel, dbscan,
        RColorBrewer, lattice, xtable, foreign, igraph, RSpectra,
        knitr, classInt, tmap, spam, ggplot2, rmarkdown, tinytest,
        rgeoda (>= 0.0.11.1), mipfp, Guerry, codingMatrices
URL: https://github.com/r-spatial/spdep/,
        https://r-spatial.github.io/spdep/
BugReports: https://github.com/r-spatial/spdep/issues/
Description: A collection of functions to create spatial weights matrix
  objects from polygon 'contiguities', from point patterns by distance and
  tessellations, for summarizing these objects, and for permitting their
  use in spatial data analysis, including regional aggregation by minimum
  spanning tree; a collection of tests for spatial 'autocorrelation',
  including global 'Morans I' and 'Gearys C' proposed by 'Cliff' and 'Ord'
  (1973, ISBN: 0850860369) and (1981, ISBN: 0850860814), 'Hubert/Mantel'
  general cross product statistic, Empirical Bayes estimates and
  'Assunção/Reis' (1999) <doi:10.1002/(SICI)1097-0258(19990830)18:16%3C2147::AID-SIM179%3E3.0.CO;2-I> Index, 'Getis/Ord' G ('Getis' and 'Ord' 1992)
  <doi:10.1111/j.1538-4632.1992.tb00261.x> and multicoloured
  join count statistics, 'APLE' ('Li et al.' )
  <doi:10.1111/j.1538-4632.2007.00708.x>, local 'Moran's I', 'Gearys C' 
  ('Anselin' 1995) <doi:10.1111/j.1538-4632.1995.tb00338.x> and
  'Getis/Ord' G ('Ord' and 'Getis' 1995)
  <doi:10.1111/j.1538-4632.1995.tb00912.x>,
  'saddlepoint' approximations ('Tiefelsdorf' 2002)
  <doi:10.1111/j.1538-4632.2002.tb01084.x> and exact tests
  for global and local 'Moran's I' ('Bivand et al.' 2009)
  <doi:10.1016/j.csda.2008.07.021> and 'LOSH' local indicators
  of spatial heteroscedasticity ('Ord' and 'Getis')
  <doi:10.1007/s00168-011-0492-y>. The implementation of most of
  these measures is described in 'Bivand' and 'Wong' (2018)
  <doi:10.1007/s11749-018-0599-x>, with further extensions in 'Bivand' (2022)
  <doi:10.1111/gean.12319>. 'Lagrange' multiplier tests for spatial dependence
  in linear models are provided ('Anselin et al'. 1996)
  <doi:10.1016/0166-0462(95)02111-6>, as are 'Rao' score tests for hypothesised
  spatial 'Durbin' models based on linear models ('Koley' and 'Bera' 2023)
  <doi:10.1080/17421772.2023.2256810>. Additions in 2024 include Local
  Indicators for Categorical Data based on 'Carrer et al.' (2021)
  <doi:10.1016/j.jas.2020.105306> and 'Bivand et al.' (2017)
  <doi:10.1016/j.spasta.2017.03.003>; also Weighted Multivariate Spatial
  Autocorrelation Measures ('Bavaud' 2024) <doi:10.1111/gean.12390>.
  <doi:10.1080/17421772.2023.2256810>. A local indicators for categorical data
  (LICD) implementation based on 'Carrer et al.' (2021) 
  <doi:10.1016/j.jas.2020.105306> and 'Bivand et al.' (2017) 
  <doi:10.1016/j.spasta.2017.03.003> was added in 1.3-7. Multivariate
  'spatialdelta' ('Bavaud' 2024) <doi:10.1111/gean.12390> was added in 1.3-13 ('Bivand' 2025 <doi:10.26034/la.cdclsl.2025.8343>).
  From 'spdep' and 'spatialreg' versions >= 1.2-1, the model fitting functions
  previously present in this package are defunct in 'spdep' and may be found
  in 'spatialreg'.
License: GPL (>= 2)
VignetteBuilder: knitr
RoxygenNote: RoxygenNote: 6.1.1
NeedsCompilation: yes
Packaged: 2026-02-13 12:34:33 UTC; rsb
Author: Roger Bivand [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-2392-6140>, ROR:
    <https://ror.org/04v53s997>),
  Micah Altman [ctb],
  Luc Anselin [ctb],
  Renato Assunção [ctb],
  Anil Bera [ctb],
  Olaf Berke [ctb],
  F. Guillaume Blanchet [ctb],
  Marilia Carvalho [ctb],
  Bjarke Christensen [ctb],
  Yongwan Chun [ctb],
  Carsten Dormann [ctb],
  Stéphane Dray [ctb],
  Dewey Dunnington [ctb] (ORCID: <https://orcid.org/0000-0002-9415-4582>),
  Virgilio Gómez-Rubio [ctb],
  Malabika Koley [ctb],
  Tomasz Kossowski [ctb] (ORCID: <https://orcid.org/0000-0002-9976-4398>),
  Elias Krainski [ctb],
  Pierre Legendre [ctb],
  Nicholas Lewin-Koh [ctb],
  Angela Li [ctb],
  Giovanni Millo [ctb],
  Werner Mueller [ctb],
  Hisaji Ono [ctb],
  Josiah Parry [ctb] (ORCID: <https://orcid.org/0000-0001-9910-865X>),
  Pedro Peres-Neto [ctb],
  Michał Pietrzak [ctb] (ORCID: <https://orcid.org/0000-0002-9263-4478>),
  Gianfranco Piras [ctb],
  Markus Reder [ctb],
  Jeff Sauer [ctb],
  Michael Tiefelsdorf [ctb],
  René Westerholt [ctb],
  Justyna Wilk [ctb] (ORCID: <https://orcid.org/0000-0003-1495-2910>),
  Levi Wolf [ctb],
  Danlin Yu [ctb]
Maintainer: Roger Bivand <Roger.Bivand@nhh.no>
Repository: CRAN
Date/Publication: 2026-02-13 13:50:02 UTC
