% Copyright 2002 by Roger S. Bivand
\encoding{UTF-8}
\name{eire}
\alias{eire}
\alias{eire.coords.utm}
\alias{eire.df}
\alias{eire.nb}
\alias{eire.polys.utm}
\docType{data}
\title{Eire data sets}
\usage{data(eire)}
\description{
The data set is now part of the spData package
}
\keyword{datasets}
